@@ -1,7 +1,7 @@
 # -*- mode: Perl -*-
 package MRTG_lib;
 ###################################################################
-# MRTG 2.15.2  Support library MRTG_lib.pm
+# MRTG 2.16.4  Support library MRTG_lib.pm
 ###################################################################
 # Created by Tobias Oetiker <tobi@oetiker.ch>
 #            and Dave Rand <dlr@bungi.com>
@@ -198,7 +198,7 @@ $VERSION = 2.100016;
        [sub{1}, sub{"Internal Error"}], #what ever the user chooses.
 
        'clonedirectory[]' =>
-       [sub{1}, sub{"Internal Error"}], #what ever the user chooses.
+       [sub{($_[0] =~ /[^,]\s*$/)}, sub{"$_[0] with comma must have the second parameter"}], 
 
        'pagetop[]' => 
        [sub{1}, sub{"Internal Error"}], #what ever the user chooses.
@@ -215,6 +215,19 @@ $VERSION = 2.100016;
        'rrdrowcount[]' => 
        [sub{1}, sub{"Internal Error"}], #what ever the user chooses.
 
+       'rrdrowcount30m[]' =>
+       [sub{1}, sub{"Internal Error"}], #what ever the user chooses.
+
+       'rrdrowcount2h[]' =>
+       [sub{1}, sub{"Internal Error"}], #what ever the user chooses.
+
+       'rrdrowcount1d[]' =>
+       [sub{1}, sub{"Internal Error"}], #what ever the user chooses.
+
+       'rrdhwrras[]' =>
+       [sub{$_[0] =~ /^RRA:(HWPREDICT|SEASONAL|DEVPREDICT|DEVSEASONAL|FAILURES):\S+(\s+RRA:(HWPREDICT|SEASONAL|DEVPREDICT|DEVSEASONAL|FAILURES):\S+)*$/},
+        sub{"This does not look like rrdtool HW RRAs. Check the rrdcreate manual page for inspiration. ($_[0])"}],
+
        'extension[]' =>
        [sub{1}, sub{"Internal Error"}], #what ever the user chooses.
 
@@ -301,6 +314,7 @@ $VERSION = 2.100016;
        [sub{$_[0] =~ /^(?:[-\w]+=\"[^"]*"(?:\s+|$))+$/},
         sub{"$_[0] must be XY=\"dddd\" AASD=\"kjlkj\" ... "}],
 
+
        'xzoom[]' =>
        [sub{($_[0] =~ /^[0-9]+(?:\.[0-9]+)?$/)},
         sub{"$_[0] must be a Number xxx.xxx"}],
@@ -324,6 +338,10 @@ $VERSION = 2.100016;
        [sub{($_[0] =~ /^[0-9]+(?:\.[0-9]+)?$/)},
         sub{"$_[0] must be a Number xxx.xxx"}],
 
+       'hwthreshhyst' =>
+       [sub{($_[0] =~ /^[0-9]+(?:\.[0-9]+)?$/)},
+        sub{"$_[0] must be a Number xxx.xxx"}],
+
        'threshmailserver' =>
        [sub{$_[0] && gethostbyname($_[0])}, sub{"Unknown mailserver hostname $_[0]"}],
 
@@ -360,6 +378,36 @@ $VERSION = 2.100016;
        'threshmailaddress[]' =>
        [sub{$_[0] && ($_[0] =~ /\S+\@\S+/)}, sub{"ThreshMailAddress $_[0] does not look like an email address at all"}],
 
+       'hwthreshmini[]' =>
+       [sub{1}, sub{"Internal Threshold Config Error"}],
+
+       'hwthreshmino[]' =>
+       [sub{1}, sub{"Internal Threshold Config Error"}],
+
+       'hwthreshmaxi[]' =>
+       [sub{1}, sub{"Internal Threshold Config Error"}],
+
+       'hwthreshmaxo[]' =>
+       [sub{1}, sub{"Internal Threshold Config Error"}],
+
+       'hwthreshdesc[]' =>
+       [sub{1}, sub{"Internal Threshold Config Error"}],
+
+       'hwthreshprogi[]' =>
+       [sub{$_[0] && (-e $_[0])}, sub{"Threshold program $_[0] cannot be executed"}],
+
+       'hwthreshprogo[]' =>
+       [sub{$_[0] && (-e $_[0])}, sub{"Threshold program $_[0] cannot be executed"}],
+
+       'hwthreshprogoki[]' =>
+       [sub{$_[0] && (-e $_[0])}, sub{"Threshold program $_[0] cannot be executed"}],
+
+       'hwthreshprogoko[]' =>
+       [sub{$_[0] && (-e $_[0])}, sub{"Threshold program $_[0] cannot be executed"}],
+
+       'hwthreshmailaddress[]' =>
+       [sub{$_[0] && ($_[0] =~ /\S+\@\S+/)}, sub{"ThreshMailAddress $_[0] does not look like an email address at all"}],
+
        'timestrpos[]' => 
        [sub{$_[0] =~ /^(no|[lr][ul])$/i}, sub{"Must be a string of NO, LU, RU, LL, RL"}],
 
@@ -1044,7 +1092,7 @@ sub cfgcheck ($$$$;$) {
 sub ensureSL($) {
 #  return;
   my $ref = shift;
-  return if $$ref eq "";
+  return if not $$ref;
   debug('dir',"ensure path IN:  '$$ref'");
   if (${MRTG_lib::SL} eq '\\'){
      # two slashes at the start of the string are OK
@@ -1130,7 +1178,7 @@ sub demonize_me ($) {
      if (my_type() eq 'VIO'){
         $main::Cleanfile3 = $pidfile;
 
-        print "MRTG detached. PID=".system(P_DETACH,$^X." ".$0." ".$cfgfile);
+        print "MRTG detached. PID=".system(P_DETACH(),$^X." ".$0." ".$cfgfile);
         exit;
      }
     } else {
@@ -1138,7 +1186,8 @@ sub demonize_me ($) {
            if (defined $pidfile && open(READPID, "<$pidfile")){
                if (not eof READPID) {
                    chomp(my $input = <READPID>);    # read process id in pidfile
-                   if ($input && kill 0 => $input) {# oops - the pid actually exists
+                   my ($pid) = $input =~ /^(\d+)$/; # to improve taint-safe code
+                   if ($pid && kill 0 => $pid) {# oops - the pid actually exists
                         die "ERROR: I Quit! Another copy of mrtg seems to be running. Check $pidfile\n";
                    }
                }
@@ -1764,7 +1813,7 @@ sub log2rrd ($$$) {
                            if (defined $$rcfg{'options'}{'perhour'}{$router});    
                        $rate{$t}{$s}/=60
                            if (defined $$rcfg{'options'}{'perminute'}{$router});
- 	  	       push @{$store{$t}{$s}}, int($rate{$t}{$s});
+ 	  	       push @{$store{$t}{$s}}, $rate{$t}{$s};
 		    }
 		    $mark{$s} -= $s;
 		    $rate{maxin}{$s} = 0;
@@ -1959,6 +2008,7 @@ sub setup_loghandlers ($){
 		$EventLog->Close;
 	    };
 	    $SIG{__DIE__} = sub {
+                return if $^S ; # no handler in eval
 		my $EventLog = Win32::EventLog->new('MRTG');
 		my $Msg = $_[0];
 		$Msg =~ s/\n/\r\n/g;
@@ -1975,7 +2025,6 @@ sub setup_loghandlers ($){
 	    last;
 	};
 	$SIG{__WARN__} = sub {
-	    
 	    if (open DEB, ">>$::global_logfile") {
 		print DEB timestamp." -- $_[0]";
 		close DEB;
@@ -1986,7 +2035,7 @@ sub setup_loghandlers ($){
 	
 	
 	$SIG{__DIE__} = sub {
-	    
+            return if $^S ; # no handler in eval	    	    
 	    if ( open DEB, ">>$::global_logfile") {
 		print DEB timestamp." -- $_[0]";
 		close DEB;
